#! /usr/bin/env python

from __future__ import division

import sys
import os
import time
import threading
import serial
import socket
from math import *
import objc
from Foundation import *
from AppKit import *


targetLocation = (34.106194, 117.712411)

GPS_HOST = 'localhost'      # The remote host
GPS_PORT = 2947             # The same port as used by the server
gpsSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
gpsSocket.connect((GPS_HOST, GPS_PORT))
gpsSocket.send('p\n')
data = gpsSocket.recv(1024)
robot = serial.Serial('tty.arduino', timeout=0)
robotSensors = serial.Serial('/dev/tty.usbserial-A4000Oz6', timeout=0.05)
statements = []
statementsLock = threading.Lock()
statementsAvailible = threading.Event()
pleaseMoveSound = NSSound.alloc().initWithContentsOfFile_byReference_(
            'Please move.mp3', True)



def getLocation():
    try:
        gpsSocket.send('o\n')
        data = gpsSocket.recv(1024).split()
        return [float(l) for l in data[3:4]]
    except (IndexError):
        return (0.0,0.0)

def getDistances():
    while '\n' in robotSensors.readline():
        pass
    s = ''
    while s == '' or len(s.split()) != 2 or '-' in s.split():
        s = robotSensors.readline()
        print s.split()
    distances = map(int, s.split())
    for i in range(len(distances)):
        if distances[i] < 0:
            distances[i] = 10000
    return distances

def turnWheels(direction):
    while '\n' in robot.readline():
        pass
        #print 'Eating input'
    if 'left' in direction:
        robot.write('l')
    elif 'right' in direction:
        robot.write('r')
    else:
        robot.write('c')
    timeout = 50
    while robot.readline().strip() not in ('Centered.', 'Left.', 'Right.')\
            and timeout > 0:
        #print 'Waiting for turn...'
        time.sleep(.1)
        timeout -= 1
    robot.write('s')

def consume():
    lastside = ''
    direction = ''
    timesDelayed = 0
    while True:
        newTimesDelayed = 0
        location  = getLocation()
        print 'Dist is', sqrt(sum([(c1-c2)**2
                for c1, c2 in zip(location, targetLocation)]))
        if len(location) == 2 and sqrt(sum([(c1-c2)**2
                for c1, c2 in zip(location, targetLocation)])) < 5e-6:
            os.system('say -v Princess "We\'re at the beach.  Let\'s party."')
            turnWheels('')
            turnWheels('left')
            turnWheels('right')
            turnWheels('')
            os.system('say -v Princess "Yay!"')
            print 'Target Location Reached!  Hit enter to resume.'
            raw_input()
            continue
        statementsAvailible.wait()
        statementsLock.acquire()
        statement = statements[-1]
        del statements[:]
        statementsAvailible.clear()
        statementsLock.release()
        robot.write('0') # Eat any old output from the Arduino.
        if statement.strip() == 'lost':
            gofor = -1
            if 'right' in lastside:
                direction = 'left'
            elif 'left' in lastside:
                direction = 'right'
            else:
                direction == 'center'
                gofor = 1
        else:
            direction, gofor, lastside = [s.strip()
                    for s in statement.split(',')]
            gofor = float(gofor)
        turnWheels(direction)
        distances = getDistances()
        minDist = min(distances)
        print minDist,
        if gofor < 0:
            print 'Reversing (%f)!' % gofor
            gofor = gofor * -1
            robot.write('<3')
        else:
            if minDist < 30:
                newTimesDelayed = timesDelayed + 1
                if timesDelayed <= 4: 
                    print 'Something\'s there (%f)!' % gofor
                    pleaseMoveSound.stop()
                    pleaseMoveSound.play()
                    gofor = 2
                else:
                    if distances[0] == minDist:
                        turnWheels('left')
                    else:
                        turnWheels('right')
                    gofor = 1
                    robot.write('<3')
            else:
                print 'Going forward (%f)!' % gofor
                robot.write('>3')
        time.sleep(gofor)
        robot.write('0')
        timesDelayed = newTimesDelayed

def produce():
    while True:
        line = sys.stdin.readline()
        if line == '':
            return
#        print line.strip()
        statementsLock.acquire()
        statements.append(line)
        statementsAvailible.set()
        statementsLock.release()

if __name__ == '__main__':
    consumer = threading.Thread(target=consume, name='consumer')
    consumer.setDaemon(True);
    consumer.start()
    produce()
    gpsSocket.close()
